import 'package:knt_bloc/knt_bloc.dart';
import 'package:native_admob_flutter/native_admob_flutter.dart';

import '../repository/monetized_repo.dart';
import '../service/admob_service.dart';

/// Handles generating & loading & showing [InterstitialAd] unit.
class FrequentlyAdsBloc extends BaseBloc {
  final AdmobService _admobService;
  final MonetizedRepo _monetizedRepo;

  InterstitialAd? _currentInterstitialAd;
  late int _retryLoadInterstitialAd;
  late int _interstitialAdCounter;

  /// Using this to define where ad request come from
  late String _currentAdTag;

  FrequentlyAdsBloc(this._admobService, this._monetizedRepo) {
    _resetData();
  }

  @override
  Stream<BaseState> handleEvent(BaseEvent event) async* {
    yield* super.handleEvent(event);
    switch (event.runtimeType) {
      case PrepareFrequentlyAdEvent:
        yield* _handlePrepareFrequentlyAdEvent(
            event as PrepareFrequentlyAdEvent);
        break;

      case ShowFrequentlyAdEvent:
        yield* _handleShowFrequentlyAdEvent(event as ShowFrequentlyAdEvent);
        break;

      case _LoadDoneFrequentlyAdEvent:
        yield* _handleLoadDoneFrequentlyAdEvent(
            event as _LoadDoneFrequentlyAdEvent);
        break;

      case _ReloadFrequentlyAdEvent:
        yield* _handleReloadFrequentlyAdEvent(
            event as _ReloadFrequentlyAdEvent);
        break;

      case _CloseFrequentlyAdEvent:
        yield* _handleCloseFrequentlyAdEvent(event as _CloseFrequentlyAdEvent);
        break;
    }
  }

  void _resetData() {
    _currentInterstitialAd = null;
    _retryLoadInterstitialAd = 0;
    _interstitialAdCounter = 0;
    _currentAdTag = '';
  }

  Stream<BaseState> _handlePrepareFrequentlyAdEvent(
      PrepareFrequentlyAdEvent event) async* {
    await _createAndLoadInterstitialAd();
  }

  Stream<BaseState> _handleShowFrequentlyAdEvent(
      ShowFrequentlyAdEvent event) async* {
    _currentAdTag = event.adTag;
    final purchasedIap = await _monetizedRepo.fetchPurchasedIapStatus();
    if (purchasedIap) {
      // Will not show any admobs when user has purchased
      yield ClosedFrequentlyAdState(_currentAdTag);
      return;
    }
    // Also check for frequent of admobs
    _interstitialAdCounter++;
    if (_interstitialAdCounter < _admobService.thresholdShowInterstitialAds) {
      // If counter is not reached to threshold, then admob will not be shown
      yield ClosedFrequentlyAdState(_currentAdTag);
      return;
    }
    // Otherwise load admobs and make money :D
    _interstitialAdCounter = 0;
    _retryLoadInterstitialAd = 0;
    if (_currentInterstitialAd == null ||
        _currentInterstitialAd?.isAvailable == false) {
      await _createAndLoadInterstitialAd(forceLoadAndShow: true);
      return;
    }
    yield* _handleDisplayFrequentlyAdEvent();
  }

  Stream<BaseState> _handleLoadDoneFrequentlyAdEvent(
      _LoadDoneFrequentlyAdEvent event) async* {
    yield* _handleDisplayFrequentlyAdEvent();
  }

  Stream<BaseState> _handleDisplayFrequentlyAdEvent() async* {
    await _currentInterstitialAd?.show();
    yield ClosedFrequentlyAdState(_currentAdTag);
  }

  Stream<BaseState> _handleReloadFrequentlyAdEvent(
      _ReloadFrequentlyAdEvent event) async* {
    await _createAndLoadInterstitialAd(forceLoadAndShow: event.forceShowAd);
  }

  Stream<BaseState> _handleCloseFrequentlyAdEvent(
      _CloseFrequentlyAdEvent event) async* {
    yield ClosedFrequentlyAdState(_currentAdTag);
  }

  Future<void> _createAndLoadInterstitialAd(
      {bool forceLoadAndShow = false}) async {
    _currentInterstitialAd?.dispose();
    final onLoaded = () {
      _retryLoadInterstitialAd = 0;
      if (forceLoadAndShow) {
        add(_LoadDoneFrequentlyAdEvent());
      }
    };
    final onClosed = () {
      add(_ReloadFrequentlyAdEvent());
    };
    final onLoadFailed = () {
      if (_retryLoadInterstitialAd < _admobService.maxRetryReloadAds) {
        _retryLoadInterstitialAd++;
        add(_ReloadFrequentlyAdEvent(forceShowAd: forceLoadAndShow));
        return;
      }
      // In case run out of retry, we close this ads flow
      if (forceLoadAndShow) {
        add(_CloseFrequentlyAdEvent());
      }
    };
    _currentInterstitialAd = _admobService.createInterstitialAd(
      onLoaded: onLoaded,
      onClosed: onClosed,
      onLoadFailed: onLoadFailed,
    );
    await _currentInterstitialAd!.load();
  }
}

// region EVENT
class PrepareFrequentlyAdEvent extends BaseEvent {}

class ShowFrequentlyAdEvent extends BaseEvent {
  final String adTag;

  const ShowFrequentlyAdEvent(this.adTag) : super(tag: adTag);
}

class _LoadDoneFrequentlyAdEvent extends BaseEvent {}

class _ReloadFrequentlyAdEvent extends BaseEvent {
  final bool forceShowAd;

  const _ReloadFrequentlyAdEvent({this.forceShowAd = false});
}

class _CloseFrequentlyAdEvent extends BaseEvent {}
// endregion

// region STATE
class ClosedFrequentlyAdState extends BaseState {
  final String adTag;

  const ClosedFrequentlyAdState(this.adTag) : super(tag: adTag);
}
// endregion
