import 'package:knt_bloc/knt_bloc.dart';
import 'package:native_admob_flutter/native_admob_flutter.dart';

import '../repository/monetized_repo.dart';
import '../service/admob_service.dart';

/// Handles generating & loading [BannerAdController]/[NativeAdController].
///
/// These controller will be used to create [BannerAd]/[NativeAd] widget.
class StaticAdsBloc extends BaseBloc {
  final AdmobService _admobService;
  final MonetizedRepo _monetizedRepo;

  BannerAdController? _currentBannerAdController;
  late int _retryLoadBannerAd;

  NativeAdController? _currentNativeAdController;
  late int _retryLoadNativeAd;

  StaticAdsBloc(this._admobService, this._monetizedRepo) {
    _resetData();
  }

  @override
  Stream<BaseState> handleEvent(BaseEvent event) async* {
    yield* super.handleEvent(event);
    switch (event.runtimeType) {
      case PrepareBannerAdEvent:
        yield* _handlePrepareBannerAdEvent(event as PrepareBannerAdEvent);
        break;

      case RefreshBannerAdEvent:
        yield* _handleRefreshBannerAdEvent(event as RefreshBannerAdEvent);
        break;

      case _LoadDoneBannerAdEvent:
        yield* _handleLoadDoneBannerAdEvent(event as _LoadDoneBannerAdEvent);
        break;

      case PrepareNativeAdEvent:
        yield* _handlePrepareNativeAdEvent(event as PrepareNativeAdEvent);
        break;

      case RefreshNativeAdEvent:
        yield* _handleRefreshNativeAdEvent(event as RefreshNativeAdEvent);
        break;

      case _LoadDoneNativeAdEvent:
        yield* _handleLoadDoneNativeAdEvent(event as _LoadDoneNativeAdEvent);
        break;
    }
  }

  Stream<BaseState> _handlePrepareBannerAdEvent(
      PrepareBannerAdEvent event) async* {
    await _createAndLoadBannerAd();
  }

  Stream<BaseState> _handleRefreshBannerAdEvent(
      RefreshBannerAdEvent event) async* {
    yield* _handleRefreshBannerAd();
  }

  Stream<BaseState> _handleLoadDoneBannerAdEvent(
      _LoadDoneBannerAdEvent event) async* {
    yield* _handleRefreshBannerAd();
  }

  Stream<BaseState> _handleRefreshBannerAd() async* {
    final purchasedIap = await _monetizedRepo.fetchPurchasedIapStatus();
    if (purchasedIap) {
      _resetData();
    }
    yield LoadedBannerAdState(
      bannerAdController: _currentBannerAdController,
    );
  }

  Stream<BaseState> _handlePrepareNativeAdEvent(
      PrepareNativeAdEvent event) async* {
    await _createAndLoadNativeAd();
  }

  Stream<BaseState> _handleRefreshNativeAdEvent(
      RefreshNativeAdEvent event) async* {
    yield* _handleRefreshNativeAd();
  }

  Stream<BaseState> _handleLoadDoneNativeAdEvent(
      _LoadDoneNativeAdEvent event) async* {
    yield* _handleRefreshNativeAd();
  }

  Stream<BaseState> _handleRefreshNativeAd() async* {
    final purchasedIap = await _monetizedRepo.fetchPurchasedIapStatus();
    if (purchasedIap) {
      _resetData();
    }
    yield LoadedNativeAdState(
      nativeAdController: _currentNativeAdController,
    );
  }

  void _resetData() {
    _currentBannerAdController = null;
    _retryLoadBannerAd = 0;
    _currentNativeAdController = null;
    _retryLoadNativeAd = 0;
  }

  Future<void> _createAndLoadBannerAd({bool forceLoadAndShow = true}) async {
    _currentBannerAdController?.dispose();
    final onLoaded = () {
      _retryLoadBannerAd = 0;
      if (forceLoadAndShow) {
        add(_LoadDoneBannerAdEvent());
      }
    };
    final onLoadFailed = () async {
      if (_retryLoadBannerAd < _admobService.maxRetryReloadAds) {
        _retryLoadBannerAd++;
        await _createAndLoadBannerAd();
      }
    };
    _currentBannerAdController = _admobService.createBannerAdController(
      onLoaded: onLoaded,
      onLoadFailed: onLoadFailed,
    );
    await _currentBannerAdController?.load();
  }

  Future<void> _createAndLoadNativeAd({bool forceLoadAndShow = true}) async {
    _currentNativeAdController?.dispose();
    final onLoaded = () {
      _retryLoadNativeAd = 0;
      if (forceLoadAndShow) {
        add(_LoadDoneNativeAdEvent());
      }
    };
    final onLoadFailed = () async {
      if (_retryLoadNativeAd < _admobService.maxRetryReloadAds) {
        _retryLoadNativeAd++;
        await _createAndLoadNativeAd();
      }
    };
    _currentNativeAdController = _admobService.createNativeAdController(
      onLoaded: onLoaded,
      onLoadFailed: onLoadFailed,
    );
    await _currentNativeAdController?.load();
  }
}

// region EVENT
class PrepareBannerAdEvent extends BaseEvent {}

class RefreshBannerAdEvent extends BaseEvent {}

class _LoadDoneBannerAdEvent extends BaseEvent {}

class PrepareNativeAdEvent extends BaseEvent {}

class RefreshNativeAdEvent extends BaseEvent {}

class _LoadDoneNativeAdEvent extends BaseEvent {}
// endregion

// region STATE
class LoadedBannerAdState extends BaseState {
  final BannerAdController? bannerAdController;

  const LoadedBannerAdState({this.bannerAdController});
}

class LoadedNativeAdState extends BaseState {
  final NativeAdController? nativeAdController;

  const LoadedNativeAdState({this.nativeAdController});
}
// endregion
