import 'dart:async';

import 'package:flutter_inapp_purchase/flutter_inapp_purchase.dart';
import 'package:knt_bloc/knt_bloc.dart';

import '../model/subscription_item.dart';
import '../repository/monetized_repo.dart';
import '../service/iap_service.dart';

/// Handles in-app-purchase flow.
class SubscriptionBloc extends BaseBloc {
  final IapService _iapService;
  final MonetizedRepo _monetizedRepo;

  // In-app-purchase service
  late bool _purchased;
  late int _retryConnection;
  StreamSubscription? _purchaseUpdatedSubscription;
  StreamSubscription? _purchaseErrorSubscription;

  SubscriptionBloc(this._iapService, this._monetizedRepo) {
    _initData();
    add(ConnectSubscriptionServiceEvent());
  }

  @override
  Future<void> close() async {
    await FlutterInappPurchase.instance.endConnection;
    _purchaseUpdatedSubscription?.cancel();
    _purchaseErrorSubscription?.cancel();
    return super.close();
  }

  @override
  Stream<BaseState> handleEvent(BaseEvent event) async* {
    yield* super.handleEvent(event);
    switch (event.runtimeType) {
      case ConnectSubscriptionServiceEvent:
        yield* _handleConnectSubscriptionServiceEvent(
            event as ConnectSubscriptionServiceEvent);
        break;

      case RequestSubscriptionEvent:
        yield* _handleRequestSubscriptionEvent(
            event as RequestSubscriptionEvent);
        break;

      case RestoreSubscriptionEvent:
        yield* _handleRestoreSubscriptionEvent(
            event as RestoreSubscriptionEvent);
        break;

      case _UpdatePurchaseInfoEvent:
        yield* _handleUpdatePurchaseInfoEvent(
            event as _UpdatePurchaseInfoEvent);
        break;

      case _UpdateRestoreInfoEvent:
        yield* _handleUpdateRestoreInfoEvent(event as _UpdateRestoreInfoEvent);
        break;

      case FetchListSkuEvent:
        yield* _handleFetchListSkuEvent(event as FetchListSkuEvent);
        break;
    }
  }

  @override
  Stream<BaseState> handleBaseErrorEvent(BaseErrorEvent event) async* {
    if (event.tag == (FetchListSkuEvent).toString()) {
      yield FetchedListSkuFailureState();
      return;
    }
    if (event.tag == (RestoreSubscriptionEvent).toString()) {
      yield FetchedRestoreFailureState();
      return;
    }
    if (event.tag == (RequestSubscriptionEvent).toString()) {
      yield UnPurchasedState();
      return;
    }
    yield* super.handleBaseErrorEvent(event);
  }

  Future<void> _initData() async {
    _purchased = false;
    _retryConnection = 0;
  }

  Stream<BaseState> _handleConnectSubscriptionServiceEvent(
      ConnectSubscriptionServiceEvent event) async* {
    try {
      await FlutterInappPurchase.instance.initConnection;
      // Setup callback for purchased actions
      _purchaseUpdatedSubscription?.cancel();
      _purchaseUpdatedSubscription =
          FlutterInappPurchase.purchaseUpdated.listen(
        (productItem) async {
          for (String sku in _skuList) {
            if (productItem?.productId == sku) {
              await _verifyPurchase(productItem);
              _purchased = true;
              add(_UpdatePurchaseInfoEvent());
              return;
            }
          }
        },
      );
      _purchaseErrorSubscription?.cancel();
      _purchaseErrorSubscription = FlutterInappPurchase.purchaseError.listen(
        (purchaseError) {
          _purchased = false;
          add(_UpdatePurchaseInfoEvent());
        },
      );
      // Get available subscriptions
      _fetchAvailableSubscription();
    } on Exception {
      if (_retryConnection < _iapService.maxRetryConnectBilling) {
        _retryConnection++;
        add(ConnectSubscriptionServiceEvent());
      } else {
        _purchased = false;
        add(_UpdatePurchaseInfoEvent());
      }
    }
  }

  Stream<BaseState> _handleRequestSubscriptionEvent(
      RequestSubscriptionEvent event) async* {
    yield PurchasingState();
    List<IAPItem> itemList =
        await FlutterInappPurchase.instance.getSubscriptions(_offeringSkuList);
    for (var item in itemList) {
      if (item.productId == event.subItem.sku) {
        FlutterInappPurchase.instance.requestPurchase(item.productId ?? '');
        return;
      }
    }
  }

  Stream<BaseState> _handleRestoreSubscriptionEvent(
      RestoreSubscriptionEvent event) async* {
    yield RestoringState();
    _fetchAvailableSubscription(forRestorePurpose: true);
  }

  Stream<BaseState> _handleUpdatePurchaseInfoEvent(
      _UpdatePurchaseInfoEvent event) async* {
    await _monetizedRepo.savePurchasedIapStatus(purchased: _purchased);
    if (_purchased) {
      yield PurchasedState();
      return;
    }
    yield UnPurchasedState();
  }

  Stream<BaseState> _handleUpdateRestoreInfoEvent(
      _UpdateRestoreInfoEvent event) async* {
    await _monetizedRepo.savePurchasedIapStatus(purchased: _purchased);
    if (_purchased) {
      yield FetchedRestoreSuccessState();
      return;
    }
    yield FetchedRestoreFailureState();
  }

  Stream<BaseState> _handleFetchListSkuEvent(FetchListSkuEvent event) async* {
    yield FetchingListSkuState();
    List<IAPItem> itemList = await cancelableFuture(
        FlutterInappPurchase.instance.getSubscriptions(_offeringSkuList));
    final subList = _iapService.skuConverter(itemList);
    if (subList.isEmpty) {
      throw KntException(KntIssue.undefined);
    }
    yield FetchedListSkuState(subList);
  }

  Future<void> _fetchAvailableSubscription(
      {bool forRestorePurpose = false}) async {
    final purchasedItems = await cancelableFuture(
        FlutterInappPurchase.instance.getAvailablePurchases());
    _purchased = false;
    for (PurchasedItem item in purchasedItems ?? []) {
      if (item.productId == null) {
        continue;
      }
      if (_skuList.contains(item.productId)) {
        await _verifyPurchase(item);
        _purchased = true;
        break;
      }
    }
    if (forRestorePurpose) {
      add(_UpdateRestoreInfoEvent());
    } else {
      add(_UpdatePurchaseInfoEvent());
    }
  }

  Future<void> _verifyPurchase(PurchasedItem? item) async {
    if (item == null) {
      return;
    }
    await FlutterInappPurchase.instance.finishTransaction(item);
  }

  List<String> get _skuList => _iapService.skuList;

  List<String> get _offeringSkuList => _iapService.offeringSkuList;
}

// region EVENT
class ConnectSubscriptionServiceEvent extends BaseEvent {}

class RequestSubscriptionEvent extends BaseEvent {
  final SubscriptionItem subItem;

  const RequestSubscriptionEvent(this.subItem);

  @override
  String toString() => 'RequestSubscriptionEvent: item has sku ${subItem.sku}';
}

class RestoreSubscriptionEvent extends BaseEvent {}

class _UpdatePurchaseInfoEvent extends BaseEvent {}

class _UpdateRestoreInfoEvent extends BaseEvent {}

class FetchListSkuEvent extends BaseEvent {}
// endregion

// region STATE
class PurchasingState extends BaseState {}

class RestoringState extends BaseState {}

class PurchasedState extends BaseState {}

class UnPurchasedState extends BaseState {}

class FetchingListSkuState extends BaseState {}

class FetchedListSkuState extends BaseHasResultState<List<SubscriptionItem>> {
  const FetchedListSkuState(List<SubscriptionItem> subList) : super(subList);
}

class FetchedListSkuFailureState extends BaseErrorState {
  FetchedListSkuFailureState()
      : super(exception: KntException(KntIssue.undefined));
}

class FetchedRestoreSuccessState extends BaseState {}

class FetchedRestoreFailureState extends BaseState {}
// endregion
