import 'package:app_tracking_transparency/app_tracking_transparency.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:native_admob_flutter/native_admob_flutter.dart';

import '../model/admob_config.dart';

/// Service handles admob initialization & generates ad unit
class AdmobService {
  final AdmobConfig _admobConfig;

  const AdmobService(this._admobConfig);

  /// Initialize [MobileAds], must be called before [runApp].
  ///
  /// For example:
  /// ```dart
  /// Future<void> main() async {
  ///   WidgetsFlutterBinding.ensureInitialized();
  ///   await getIt<AdmobService>().init();
  ///   runApp(App());
  /// }
  /// ```
  Future<void> init() async {
    // Need to request tracking app
    try {
      final trackingStatus =
          await AppTrackingTransparency.trackingAuthorizationStatus;
      if (trackingStatus == TrackingStatus.notDetermined) {
        await AppTrackingTransparency.requestTrackingAuthorization();
      }
    } on PlatformException {}

    // If success, safely initialize admob
    await MobileAds.initialize(
      nativeAdUnitId: nativeAdUnitId,
      bannerAdUnitId: bannerAdUnitId,
      interstitialAdUnitId: interstitialAdUnitId,
    );
    await MobileAds.setAppVolume(appVolume ?? 0.0);
    await MobileAds.setTestDeviceIds(testDeviceIds);
  }

  /// Generate [InterstitialAd] with callback functions.
  InterstitialAd createInterstitialAd({
    Function()? onLoaded,
    Function()? onClosed,
    Function()? onLoadFailed,
  }) {
    final ad = InterstitialAd();
    ad.onEvent.listen(
      (e) {
        final event = e.keys.first;
        loggingBuilder?.call('Interstitial Ad Event $event');
        switch (event) {
          case FullScreenAdEvent.loaded:
            onLoaded?.call();
            break;
          case FullScreenAdEvent.closed:
            onClosed?.call();
            break;
          case FullScreenAdEvent.loadFailed:
            onLoadFailed?.call();
            loggingBuilder?.call('Interstitial Ad load failed error'
                ' - ${e.values.first.toString()}');
            break;
          default:
            break;
        }
      },
      onError: (error) {
        loggingBuilder?.call('Interstitial Ad error $error');
      },
    );
    return ad;
  }

  /// Generate [BannerAdController] with callback functions, then use this controller to create [BannerAd] widget.
  ///
  /// For example:
  ///
  /// ```dart
  /// BannerAd(
  ///   controller: controller,
  ///   builder: (context, child) {
  ///     return Container(
  ///       color: Colors.black,
  ///       child: child,
  ///     );
  ///   },
  ///   size: BannerSize.ADAPTIVE,
  /// )
  /// ```
  BannerAdController createBannerAdController({
    Function()? onLoaded,
    Function()? onLoadFailed,
  }) {
    final controller = BannerAdController();
    controller.onEvent.listen(
      (e) {
        final event = e.keys.first;
        loggingBuilder?.call('Banner Ad Event $event');
        switch (event) {
          case BannerAdEvent.loaded:
            onLoaded?.call();
            break;
          case BannerAdEvent.loadFailed:
            onLoadFailed?.call();
            loggingBuilder?.call(
                'Banner Ad load failed error - ${e.values.first.toString()}');
            break;
          default:
            break;
        }
      },
      onError: (error) {
        loggingBuilder?.call('Banner Ad error $error');
      },
    );
    return controller;
  }

  /// Generate [NativeAdController] with callback functions, then use this controller to create [NativeAd] widget.
  NativeAdController createNativeAdController({
    Function()? onLoaded,
    Function()? onLoadFailed,
  }) {
    final controller = NativeAdController();
    controller.onEvent.listen(
      (e) {
        final event = e.keys.first;
        loggingBuilder?.call('Native Ad Event $event');
        switch (event) {
          case NativeAdEvent.loaded:
            onLoaded?.call();
            break;
          case NativeAdEvent.loadFailed:
            onLoadFailed?.call();
            loggingBuilder?.call(
                'Native Ad load failed error - ${e.values.first.toString()}');
            break;
          default:
            break;
        }
      },
      onError: (error) {
        loggingBuilder?.call('Native Ad error $error');
      },
    );
    return controller;
  }

  String? get nativeAdUnitId => _admobConfig.nativeAdUnitId;

  String? get bannerAdUnitId => _admobConfig.bannerAdUnitId;

  String? get interstitialAdUnitId => _admobConfig.interstitialAdUnitId;

  double? get appVolume => _admobConfig.appVolume;

  List<String> get testDeviceIds => _admobConfig.testDeviceIds;

  Function(String)? get loggingBuilder => _admobConfig.loggingBuilder;

  int get thresholdShowInterstitialAds =>
      _admobConfig.thresholdShowInterstitialAds;

  int get maxRetryReloadAds => _admobConfig.maxRetryReloadAds;
}
